/*
* Line follower wannabe
* PM - proiect
*/

#define F_CPU 16000000
#define PRAG 340
#define SPEED 300

#include <avr/io.h>
#include <avr/interrupt.h>

void ADC_init(void)
{
	ADMUX = 0;
	ADMUX = (1 << REFS0);
	
	ADCSRA = (1 << ADEN) | (7 << ADPS0);
}

int ADC_get(uint8_t channel)
{

	ADMUX &= ~(0b00011111 << MUX0); 
	ADMUX |= channel; 

		
	ADCSRA |= (1 << ADSC);	//start conversion
	while(ADCSRA & (1 << ADSC));	//wait for conversion to finish

	return ADC;
}


void init()
{

	/* input for sensors */
	DDRA = 0x00;
	PORTA = 0xFF;

	/* port D output*/
	DDRD = 0xFF;
	PORTD = 0x00;

	/* PWM */
	PORTD &= ~(1 << PD4);
	PORTD &= ~(1 << PD5);


	TCCR1A |= (1 << WGM11) | (1 << COM1A1) | (1 << COM1B1);
	TCCR1B |= (1 << WGM13) | (1 << WGM12) | (1 << CS12);
	
	ICR1 = 1024;

	/* motors setup */
        PORTD |= (1 << PD1);
        PORTD &= ~(1 << PD0);
        PORTD |= (1 << PD3);
        PORTD &= ~(1 << PD2);

}

/* set motors */
void move(int speedLeft , int speedRight)
{
	/* left motor */
	OCR1A = speedLeft;

	/* right motor */
	OCR1B = speedRight;
}

int main(void) {
	
	init();
	ADC_init();
	
	move(SPEED,SPEED);

	/* check if any sensor is on line */
	int i,on_line = 0;
	
	/*no. of sensors on the left/right side */
	int no_left_sensors_on_line = 0;
	int no_right_sensors_on_line = 0;

	while(1) {
		on_line = 0;
		no_left_sensors_on_line = 0;
		no_right_sensors_on_line = 0;

		for (i = 0; i < 4; i++) {
			int adc = ADC_get(i);

			if (adc > PRAG) {
				no_right_sensors_on_line++;
				on_line = 1;
			} 
		}
		for (i = 4; i < 8; i++) {
			int adc = ADC_get(i);

			if (adc > PRAG) {
				no_left_sensors_on_line++;
				on_line = 1;
			} 
		}

		/* check for no sensor on line */
		if(on_line == 0) {
			move(0,0);
		}
		else {
			if(no_left_sensors_on_line == no_right_sensors_on_line){
				move(SPEED,SPEED);
			}
			if (no_left_sensors_on_line > no_right_sensors_on_line) {
				if (no_left_sensors_on_line - no_right_sensors_on_line == 4) {
					move(SPEED,0);
				}

				if (no_left_sensors_on_line - no_right_sensors_on_line == 3) {
					if (no_left_sensors_on_line == 3){
						move(SPEED * 0.9, 0);
					}
					if (no_left_sensors_on_line == 4){
						move(SPEED * 0.8, 0);
					}
				}

				if (no_left_sensors_on_line - no_right_sensors_on_line == 2) {
					if (no_left_sensors_on_line == 4) {
						move(SPEED * 0.9, 0);
					}
					if (no_left_sensors_on_line == 3) {
						move(SPEED * 0.8, 0);
					}
					if (no_left_sensors_on_line == 2) {
						move(SPEED * 0.7, 0);
					}
				}

				if (no_left_sensors_on_line - no_right_sensors_on_line == 1) {
					if (no_left_sensors_on_line == 4) {
						move(SPEED * 0.9, 0);
					}
					if (no_left_sensors_on_line == 3) {
						move(SPEED * 0.85, 0);
					}
					if (no_left_sensors_on_line == 2) {
						move(SPEED * 0.8, 0);
					}
					if (no_left_sensors_on_line == 1) {
						move(SPEED * 0.7, 0);
					}
				}
			}

			if (no_left_sensors_on_line < no_right_sensors_on_line) {
				if (no_left_sensors_on_line - no_right_sensors_on_line == -4) {
					move(0, SPEED);
				}

				if (no_left_sensors_on_line - no_right_sensors_on_line == -3) {
					if (no_right_sensors_on_line == 3){
						move(0, SPEED * 0.9);
					}
					if (no_right_sensors_on_line == 4){
						move(0, SPEED * 0.8);
					}
				}

				if (no_left_sensors_on_line - no_right_sensors_on_line == -2) {
					if (no_right_sensors_on_line == 4) {
						move(0, SPEED * 0.9);
					}
					if (no_right_sensors_on_line == 3) {
						move(0, SPEED * 0.8);
					}
					if (no_right_sensors_on_line == 2) {
						move(0, SPEED * 0.7);
					}
				}

				if (no_left_sensors_on_line - no_right_sensors_on_line == 1) {
					if (no_right_sensors_on_line == 4) {
						move(0, SPEED * 0.9);
					}
					if (no_right_sensors_on_line == 3) {
						move(0, SPEED * 0.85);
					}
					if (no_right_sensors_on_line == 2) {
						move(0, SPEED * 0.8);
					}
					if (no_right_sensors_on_line == 1) {
						move(0, SPEED * 0.7);
					}
				}
			}
		}

	}
	return 0;
}
